@@ -10,6 +10,20 @@
 void InitCallbackManager(void);
 void DeleteCallbackManager(void);
 
+typedef struct _CallbackRec {
+    CallbackProcPtr proc;
+    void *data;
+    Bool deleted;
+    struct _CallbackRec *next;
+} CallbackRec, *CallbackPtr;
+
+typedef struct _CallbackList {
+    int inCallback;
+    Bool deleted;
+    int numDeleted;
+    CallbackPtr list;
+} CallbackListRec;
+
 /*
  * @brief delete a callback list
  *
