@@ -21,7 +21,7 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-#include <dix-config.h>
+#include <kdrive-config.h>
 #include <xkb-config.h>
 #include "kdrive.h"
 #include "inputstr.h"
@@ -32,6 +32,7 @@
 #include <X11/XF86keysym.h>
 #endif
 #include <stdio.h>
+#include <signal.h>
 #include <sys/file.h>           /* needed for FNONBLOCK & FASYNC */
 
 #include <X11/extensions/XI.h>
@@ -39,6 +40,8 @@
 
 #include "config/hotplug_priv.h"
 #include "dix/input_priv.h"
+#include "dix/inpututils_priv.h"
+#include "dix/screenint_priv.h"
 #include "mi/mi_priv.h"
 #include "mi/mipointer_priv.h"
 #include "os/cmdline.h"
@@ -49,9 +52,17 @@
 #include "exglobals.h"
 #include "eventstr.h"
 #include "xserver-properties.h"
-#include "inpututils.h"
 #include "optionstr.h"
 
+#if defined(CONFIG_UDEV) || defined(CONFIG_HAL)
+#include "config/hotplug_priv.h"
+#endif
+
+#ifdef KDRIVE_EVDEV
+#define DEV_INPUT_EVENT_PREFIX "/dev/input/event"
+#define DEV_INPUT_EVENT_PREFIX_LEN (sizeof(DEV_INPUT_EVENT_PREFIX) - 1)
+#endif
+
 #define AtomFromName(x) MakeAtom(x, strlen(x), 1)
 
 struct KdConfigDevice {
@@ -77,6 +88,22 @@ static KdPointerMatrix kdPointerMatrix = {
      {0, 1, 0}}
 };
 
+#define KD_MAX_INPUT_FDS    8
+
+typedef struct _kdInputFd {
+    int fd;
+    void (*read) (int fd, void *closure);
+    int (*enable) (int fd, void *closure);
+    void (*disable) (int fd, void *closure);
+    void *closure;
+} KdInputFd;
+
+static KdInputFd kdInputFds[KD_MAX_INPUT_FDS];
+static int kdNumInputFds = 0;
+#ifdef KDRIVE_KBD
+static int kdnFds = 0;
+#endif
+
 extern Bool kdRawPointerCoordinates;
 
 extern const char *kdGlobalXkbRules;
@@ -85,6 +112,51 @@ extern const char *kdGlobalXkbLayout;
 extern const char *kdGlobalXkbVariant;
 extern const char *kdGlobalXkbOptions;
 
+#ifdef KDRIVE_KBD
+static void KdSigio(int sig)
+{
+    for (int i = 0; i < kdNumInputFds; i++)
+        (*kdInputFds[i].read) (kdInputFds[i].fd, kdInputFds[i].closure);
+}
+
+static void KdBlockSigio(void)
+{
+    sigset_t set;
+
+    sigemptyset(&set);
+    sigaddset(&set, SIGIO);
+    sigprocmask(SIG_BLOCK, &set, 0);
+}
+
+static void KdUnblockSigio(void)
+{
+    sigset_t set;
+
+    sigemptyset(&set);
+    sigaddset(&set, SIGIO);
+    sigprocmask(SIG_UNBLOCK, &set, 0);
+}
+
+#undef VERIFY_SIGIO
+#ifdef VERIFY_SIGIO
+
+void KdAssertSigioBlocked(char *where)
+{
+    sigset_t set, old;
+
+    sigemptyset(&set);
+    sigprocmask(SIG_BLOCK, &set, &old);
+    if (!sigismember(&old, SIGIO))
+        ErrorF("SIGIO not blocked at %s\n", where);
+}
+
+#else
+
+#define KdAssertSigioBlocked(s)
+
+#endif
+#endif
+
 #ifdef FNONBLOCK
 #define NOBLOCK FNONBLOCK
 #else
@@ -102,13 +174,119 @@ KdResetInputMachine(void)
     }
 }
 
+static void KdNonBlockFd(int fd)
+{
+#ifndef WIN32
+    int flags = fcntl(fd, F_GETFL);
+    flags |= FASYNC | NOBLOCK;
+    fcntl(fd, F_SETFL, flags);
+#endif
+}
+
+static void KdNotifyFd(int fd, int ready, void *data)
+{
+    int i = (int) (intptr_t) data;
+    (*kdInputFds[i].read)(fd, kdInputFds[i].closure);
+}
+
+static void KdAddFd(int fd, int i)
+{
+#ifdef KDRIVE_KBD
+    struct sigaction act;
+
+    sigset_t set;
+
+    kdnFds++;
+    fcntl(fd, F_SETOWN, getpid());
+#endif
+    KdNonBlockFd(fd);
+    InputThreadRegisterDev(fd, KdNotifyFd, (void *) (intptr_t) i);
+#ifdef KDRIVE_KBD
+/*  AddEnabledDevice(fd); */
+    memset(&act, '\0', sizeof act);
+    act.sa_handler = KdSigio;
+
+    sigemptyset(&act.sa_mask);
+    sigaddset(&act.sa_mask, SIGIO);
+    sigaddset(&act.sa_mask, SIGALRM);
+    sigaddset(&act.sa_mask, SIGVTALRM);
+    sigaction(SIGIO, &act, 0);
+    sigemptyset(&set);
+    sigprocmask(SIG_SETMASK, &set, 0);
+#endif
+}
+
+static void KdRemoveFd(int fd)
+{
+    InputThreadUnregisterDev(fd);
+#ifndef WIN32
+    int flags = fcntl(fd, F_GETFL);
+    flags &= ~(FASYNC | NOBLOCK);
+    fcntl(fd, F_SETFL, flags);
+#endif
+#ifdef KDRIVE_KBD
+    struct sigaction act;
+    kdnFds--;
+    if (kdnFds == 0) {
+        memset(&act, '\0', sizeof act);
+        act.sa_handler = SIG_IGN;
+        sigemptyset(&act.sa_mask);
+        sigaction(SIGIO, &act, 0);
+    }
+#endif
+}
+
+Bool KdRegisterFd(int fd, void (*read) (int fd, void *closure), void *closure)
+{
+    if (kdNumInputFds == KD_MAX_INPUT_FDS)
+        return FALSE;
+    kdInputFds[kdNumInputFds].fd = fd;
+    kdInputFds[kdNumInputFds].read = read;
+    kdInputFds[kdNumInputFds].enable = 0;
+    kdInputFds[kdNumInputFds].disable = 0;
+    kdInputFds[kdNumInputFds].closure = closure;
+    if (kdInputEnabled)
+        KdAddFd(fd, kdNumInputFds);
+    kdNumInputFds++;
+    return TRUE;
+}
+
+void KdUnregisterFd(void *closure, int fd, Bool do_close)
+{
+    int i, j;
+
+    for (i = 0; i < kdNumInputFds; i++) {
+        if (kdInputFds[i].closure == closure &&
+            (fd == -1 || kdInputFds[i].fd == fd)) {
+            if (kdInputEnabled)
+                KdRemoveFd(kdInputFds[i].fd);
+            if (do_close)
+                close(kdInputFds[i].fd);
+            for (j = i; j < (kdNumInputFds - 1); j++)
+                kdInputFds[j] = kdInputFds[j + 1];
+            kdNumInputFds--;
+            break;
+        }
+    }
+}
+
+void KdUnregisterFds(void *closure, Bool do_close)
+{
+    KdUnregisterFd(closure, -1, do_close);
+}
+
 void
 KdDisableInput(void)
 {
     KdKeyboardInfo *ki;
     KdPointerInfo *pi;
+    int found = 0, i = 0;
 
+#ifndef KDRIVE_KBD
     input_lock();
+#else
+    KdBlockSigio();
+#endif
 
     for (ki = kdKeyboards; ki; ki = ki->next) {
         if (ki->driver && ki->driver->Disable)
@@ -120,6 +298,49 @@ KdDisableInput(void)
             (*pi->driver->Disable) (pi);
     }
 
+    if (kdNumInputFds) {
+        ErrorF("[KdDisableInput] Buggy drivers: still %d input fds left!",
+               kdNumInputFds);
+        i = 0;
+        while (i < kdNumInputFds) {
+            found = 0;
+            for (ki = kdKeyboards; ki; ki = ki->next) {
+                if (ki == kdInputFds[i].closure) {
+                    ErrorF("    fd %d belongs to keybd driver %s\n",
+                           kdInputFds[i].fd,
+                           ki->driver && ki->driver->name ?
+                           ki->driver->name : "(unnamed!)");
+                    found = 1;
+                    break;
+                }
+            }
+
+            if (found) {
+                i++;
+                continue;
+            }
+
+            for (pi = kdPointers; pi; pi = pi->next) {
+                if (pi == kdInputFds[i].closure) {
+                    ErrorF("    fd %d belongs to pointer driver %s\n",
+                           kdInputFds[i].fd,
+                           pi->driver && pi->driver->name ?
+                           pi->driver->name : "(unnamed!)");
+                    break;
+                }
+            }
+
+            if (found) {
+                i++;
+                continue;
+            }
+
+            ErrorF("    fd %d not claimed by any active device!\n",
+                   kdInputFds[i].fd);
+            KdUnregisterFd(kdInputFds[i].closure, kdInputFds[i].fd, TRUE);
+        }
+    }
+
     kdInputEnabled = FALSE;
 }
 
@@ -148,7 +369,11 @@ KdEnableInput(void)
         NoticeEventTime (&ev, pi->dixdev);
     }
 
+#ifndef KDRIVE_KBD
     input_unlock();
+#else
+    KdUnblockSigio();
+#endif
 }
 
 static KdKeyboardDriver *
@@ -351,8 +576,7 @@ KdPointerProc(DeviceIntPtr pDevice, int onoff)
     return BadImplementation;
 }
 
-static void
-KdRingBell(KdKeyboardInfo * ki, int volume, int pitch, int duration)
+void KdRingBell(KdKeyboardInfo * ki, int volume, int pitch, int duration)
 {
     if (!ki || !ki->driver || !ki->driver->Bell)
         return;
@@ -383,6 +607,11 @@ DDXRingBell(int volume, int pitch, int duration)
 {
     KdKeyboardInfo *ki = NULL;
 
+    if (kdOsFuncs->Bell) {
+        kdOsFuncs->Bell(volume, pitch, duration);
+        return;
+    }
+
     for (ki = kdKeyboards; ki; ki = ki->next) {
         if (ki->dixdev->coreEvents)
             KdRingBell(ki, volume, pitch, duration);
@@ -401,8 +630,7 @@ KdSetLeds(KdKeyboardInfo * ki, int leds)
     }
 }
 
-static void
-KdSetLed(KdKeyboardInfo * ki, int led, Bool on)
+void KdSetLed(KdKeyboardInfo * ki, int led, Bool on)
 {
     if (!ki || !ki->dixdev || !ki->dixdev->kbdfeed)
         return;
@@ -465,6 +693,20 @@ KdComputePointerMatrix(KdPointerMatrix * m, Rotation randr, int width,
     }
 }
 
+void
+KdScreenToPointerCoords(int *x, int *y)
+{
+    int (*m)[3] = kdPointerMatrix.matrix;
+    int div = m[0][1] * m[1][0] - m[1][1] * m[0][0];
+    int sx = *x;
+    int sy = *y;
+
+    *x = (m[0][1] * sy - m[0][1] * m[1][2] + m[1][1] * m[0][2] -
+          m[1][1] * sx) / div;
+    *y = (m[1][0] * sx + m[0][0] * m[1][2] - m[1][0] * m[0][2] -
+          m[0][0] * sy) / div;
+}
+
 static void
 KdKbdCtrl(DeviceIntPtr pDevice, KeybdCtrl * ctrl)
 {
@@ -532,6 +774,10 @@ KdKeyboardProc(DeviceIntPtr pDevice, int onoff)
             return BadImplementation;
         }
 
+        if (ki->driver->PreInit) {
+            (*ki->driver->PreInit)(ki);
+        }
+
         memset(&rmlvo, 0, sizeof(rmlvo));
         rmlvo.rules = ki->xkbRules;
         rmlvo.model = ki->xkbModel;
@@ -923,10 +1169,19 @@ KdParseKbdOptions(KdKeyboardInfo * ki)
             ErrorF("Kbd option key (%s) of value (%s) not assigned!\n",
                    key, value);
     }
+
+#ifdef KDRIVE_EVDEV
+    if (!ki->driver && ki->path != NULL &&
+        strncasecmp(ki->path,
+                    DEV_INPUT_EVENT_PREFIX,
+                    DEV_INPUT_EVENT_PREFIX_LEN) == 0) {
+            ki->driver = KdFindKeyboardDriver("evdev");
+            ki->options = input_option_new(ki->options, "driver", "evdev");
+    }
+#endif
 }
 
-static KdKeyboardInfo *
-KdParseKeyboard(const char *arg)
+KdKeyboardInfo *KdParseKeyboard(const char *arg)
 {
     char save[1024];
     char delim;
@@ -1033,6 +1288,16 @@ KdParsePointerOptions(KdPointerInfo * pi)
             ErrorF("Pointer option key (%s) of value (%s) not assigned!\n",
                    key, value);
     }
+
+#ifdef KDRIVE_EVDEV
+    if (!pi->driver && pi->path != NULL &&
+        strncasecmp(pi->path,
+                    DEV_INPUT_EVENT_PREFIX,
+                    DEV_INPUT_EVENT_PREFIX_LEN) == 0) {
+            pi->driver = KdFindPointerDriver("evdev");
+            pi->options = input_option_new(pi->options, "driver", "evdev");
+    }
+#endif
 }
 
 /*
@@ -1045,9 +1310,7 @@ KdParsePointerOptions(KdPointerInfo * pi)
  *      2button     emulate middle button
  *      {NMO}       Reorder buttons
  */
-
-static KdPointerInfo *
-KdParsePointer(const char *arg)
+KdPointerInfo *KdParsePointer(const char *arg)
 {
     char save[1024];
     char delim;
@@ -1130,11 +1393,27 @@ KdInitInput(void)
     KdKeyboardInfo *ki;
     struct KdConfigDevice *dev;
 
+#ifndef KDRIVE_KBD
     if (kdConfigPointers || kdConfigKeyboards)
         InputThreadPreInit();
+#else
+    InputThreadEnable = FALSE;
+#endif
 
     kdInputEnabled = TRUE;
 
+#ifdef KDRIVE_KBD
+    if (!kdConfigKeyboards) {
+        KdAddConfigKeyboard("keyboard");
+    }
+#endif
+
+#ifdef KDRIVE_MOUSE
+    if (!kdConfigPointers) {
+        KdAddConfigPointer("mouse");
+    }
+#endif
+
     for (dev = kdConfigPointers; dev; dev = dev->next) {
         pi = KdParsePointer(dev->line);
         if (!pi)
@@ -1522,9 +1801,6 @@ KdClassifyInput(KdPointerInfo * pi, int type, int x, int y, int z, int b)
     return keyboard;
 }
 
-static void
-_KdEnqueuePointerEvent(KdPointerInfo * pi, int type, int x, int y, int z,
-                       int b, int absrel, Bool force);
 /* We return true if we're stealing the event. */
 static Bool
 KdRunMouseMachine(KdPointerInfo * pi, KdInputClass c, int type, int x, int y,
@@ -1598,9 +1874,8 @@ KdHandlePointerEvent(KdPointerInfo * pi, int type, int x, int y, int z, int b,
     return FALSE;
 }
 
-static void
-_KdEnqueuePointerEvent(KdPointerInfo * pi, int type, int x, int y, int z,
-                       int b, int absrel, Bool force)
+void _KdEnqueuePointerEvent(KdPointerInfo * pi, int type, int x, int y, int z,
+                            int b, int absrel, Bool force)
 {
     int valuators[3] = { x, y, z };
     ValuatorMask mask;
@@ -1620,7 +1895,35 @@ KdReceiveTimeout(KdPointerInfo * pi)
     KdRunMouseMachine(pi, timeout, 0, 0, 0, 0, 0, 0);
 }
 
-extern int nClients;
+void
+KdReleaseAllKeys(void)
+{
+#if 0
+    int key;
+    KdKeyboardInfo *ki;
+
+#ifndef KDRIVE_KBD
+    input_lock();
+#else
+    KdBlockSigio();
+#endif
+
+    for (ki = kdKeyboards; ki; ki = ki->next) {
+        for (key = ki->keySyms.minKeyCode; key < ki->keySyms.maxKeyCode; key++) {
+            if (key_is_down(ki->dixdev, key, KEY_POSTED | KEY_PROCESSED)) {
+                KdHandleKeyboardEvent(ki, KeyRelease, key);
+                QueueGetKeyboardEvents(ki->dixdev, KeyRelease, key, NULL);
+            }
+        }
+    }
+
+#ifndef KDRIVE_KBD
+    input_unlock();
+#else
+    KdUnblockSigio();
+#endif
+#endif
+}
 
 static void
 KdCheckLock(void)
@@ -1772,6 +2075,11 @@ KdBlockHandler(ScreenPtr pScreen, void *timeo)
                 myTimeout = ms;
         }
     }
+    /* if we need to poll for events, do that */
+    if (kdOsFuncs->pollEvents) {
+        kdOsFuncs->pollEvents();
+        myTimeout = 20;
+    }
     if (myTimeout > 0)
         AdjustWaitForDelay(timeo, myTimeout);
 }
@@ -1785,12 +2093,22 @@ KdWakeupHandler(ScreenPtr pScreen, int result)
         if (pi->timeoutPending) {
             if ((long) (GetTimeInMillis() - pi->emulationTimeout) >= 0) {
                 pi->timeoutPending = FALSE;
+#ifndef KDRIVE_KBD
                 input_lock();
+#else
+                KdBlockSigio();
+#endif
                 KdReceiveTimeout(pi);
+#ifndef KDRIVE_KBD
                 input_unlock();
+#else
+                KdUnblockSigio();
+#endif
             }
         }
     }
+    if (kdSwitchPending)
+        KdProcessSwitch();
 }
 
 #define KdScreenOrigin(pScreen) (&(KdGetScreenPriv(pScreen)->screen->origin))
@@ -1799,14 +2117,11 @@ static Bool
 KdCursorOffScreen(ScreenPtr *ppScreen, int *x, int *y)
 {
     ScreenPtr pScreen = *ppScreen;
-    ScreenPtr pNewScreen;
-    int n;
-    int dx, dy;
     int best_x, best_y;
     int n_best_x, n_best_y;
     CARD32 ms;
 
-    if (kdDisableZaphod || screenInfo.numScreens <= 1)
+    if (kdDisableZaphod || (!dixScreenExists(1)))
         return FALSE;
 
     if (0 <= *x && *x < pScreen->width && 0 <= *y && *y < pScreen->height)
@@ -1821,42 +2136,44 @@ KdCursorOffScreen(ScreenPtr *ppScreen, int *x, int *y)
     best_x = 32767;
     n_best_y = -1;
     best_y = 32767;
-    for (n = 0; n < screenInfo.numScreens; n++) {
-        pNewScreen = screenInfo.screens[n];
-        if (pNewScreen == pScreen)
+
+    DIX_FOR_EACH_SCREEN({
+        if (walkScreen == pScreen)
             continue;
-        dx = KdScreenOrigin(pNewScreen)->x - KdScreenOrigin(pScreen)->x;
-        dy = KdScreenOrigin(pNewScreen)->y - KdScreenOrigin(pScreen)->y;
+        int dx = KdScreenOrigin(walkScreen)->x - KdScreenOrigin(pScreen)->x;
+        int dy = KdScreenOrigin(walkScreen)->y - KdScreenOrigin(pScreen)->y;
         if (*x < 0) {
             if (dx < 0 && -dx < best_x) {
                 best_x = -dx;
-                n_best_x = n;
+                n_best_x = walkScreenIdx;
             }
         }
         else if (*x >= pScreen->width) {
             if (dx > 0 && dx < best_x) {
                 best_x = dx;
-                n_best_x = n;
+                n_best_x = walkScreenIdx;
             }
         }
         if (*y < 0) {
             if (dy < 0 && -dy < best_y) {
                 best_y = -dy;
-                n_best_y = n;
+                n_best_y = walkScreenIdx;
             }
         }
         else if (*y >= pScreen->height) {
             if (dy > 0 && dy < best_y) {
                 best_y = dy;
-                n_best_y = n;
+                n_best_y = walkScreenIdx;
             }
         }
-    }
+    });
+
     if (best_y < best_x)
         n_best_x = n_best_y;
     if (n_best_x == -1)
         return FALSE;
-    pNewScreen = screenInfo.screens[n_best_x];
+
+    ScreenPtr pNewScreen = dixGetScreenPtr(n_best_x);
 
     if (*x < 0)
         *x += pNewScreen->width;
@@ -1877,12 +2194,23 @@ KdCrossScreen(ScreenPtr pScreen, Bool entering)
 {
 }
 
+int KdCurScreen;                /* current event screen */
+
 static void
 KdWarpCursor(DeviceIntPtr pDev, ScreenPtr pScreen, int x, int y)
 {
+#ifndef KDRIVE_KBD
     input_lock();
+#else
+    KdBlockSigio();
+#endif
+    KdCurScreen = pScreen->myNum;
     miPointerWarpCursor(pDev, pScreen, x, y);
+#ifndef KDRIVE_KBD
     input_unlock();
+#else
+    KdUnblockSigio();
+#endif
 }
 
 miPointerScreenFuncRec kdPointerScreenFuncs = {
@@ -1895,6 +2223,8 @@ void
 ProcessInputEvents(void)
 {
     mieqProcessInputEvents();
+    if (kdSwitchPending)
+        KdProcessSwitch();
     KdCheckLock();
 }
 
